@@ -3,10 +3,11 @@
 // (online at: https://github.com/unicode-org/icu4x/blob/main/LICENSE ).
 
 #[cfg(test)]
-#[cfg(feature = "serde")]
+#[cfg(feature = "serialize")]
 mod test;
 
-use crate::error::Error;
+use crate::buf::BufferMarker;
+use crate::error::{DataError, DataErrorKind};
 use crate::marker::DataMarker;
 use crate::resource::ResourceKey;
 use crate::resource::ResourcePath;
@@ -57,7 +58,7 @@ impl DataRequest {
     /// ```
     /// use icu_provider::prelude::*;
     ///
-    /// const FOO_BAR: ResourceKey = icu_provider::resource_key!(x, "foo", "bar", 1);
+    /// const FOO_BAR: ResourceKey = icu_provider::resource_key!("foo/bar@1");
     ///
     /// let req_no_langid = DataRequest {
     ///     resource_path: ResourcePath {
@@ -76,32 +77,32 @@ impl DataRequest {
     ///     }
     /// };
     ///
-    /// assert!(matches!(req_no_langid.try_langid(), Err(DataError::NeedsLanguageIdentifier(_))));
-    /// assert!(matches!(req_with_langid.try_langid(), Ok(_)));
+    /// assert!(matches!(
+    ///     req_no_langid.try_langid(),
+    ///     Err(DataError { kind: DataErrorKind::NeedsLocale, .. })
+    /// ));
+    /// assert!(matches!(
+    ///     req_with_langid.try_langid(),
+    ///     Ok(_)
+    /// ));
     /// ```
-    pub fn try_langid(&self) -> Result<&LanguageIdentifier, Error> {
+    pub fn try_langid(&self) -> Result<&LanguageIdentifier, DataError> {
         self.resource_path
             .options
             .langid
             .as_ref()
-            .ok_or_else(|| Error::NeedsLanguageIdentifier(self.clone()))
+            .ok_or_else(|| DataErrorKind::NeedsLocale.with_req(self))
     }
 }
 
 /// A response object containing metadata about the returned data.
 #[derive(Debug, Clone, PartialEq, Default)]
+#[non_exhaustive]
 pub struct DataResponseMetadata {
     /// The language of the returned data, or None if the resource key isn't localized.
     pub data_langid: Option<LanguageIdentifier>,
-}
-
-pub(crate) enum DataPayloadInner<'data, M>
-where
-    M: DataMarker<'data>,
-{
-    RcStruct(Yoke<M::Yokeable, Rc<M::Cart>>),
-    Owned(Yoke<M::Yokeable, ()>),
-    RcBuf(Yoke<M::Yokeable, Rc<[u8]>>),
+    /// The format of the buffer for buffer-backed data, if known (for example, JSON).
+    pub buffer_format: Option<crate::buf::BufferFormat>,
 }
 
 /// A container for data payloads returned from a [`DataProvider`].
@@ -111,8 +112,7 @@ where
 /// several data stores ("carts"):
 ///
 /// 1. Fully-owned structured data ([`DataPayload::from_owned()`])
-/// 2. Partially-owned structured data in an [`Rc`] ([`DataPayload::from_partial_owned()`])
-/// 3. A reference-counted byte buffer ([`DataPayload::try_from_rc_buffer()`])
+/// 2. A reference-counted byte buffer ([`DataPayload::try_from_rc_buffer()`])
 ///
 /// The type of the data stored in [`DataPayload`], and the type of the structured data store
 /// (cart), is determined by the [`DataMarker`] type parameter.
@@ -132,11 +132,6 @@ where
 /// To transform a [`DataPayload`] to a different type backed by the same data store (cart), use
 /// [`DataPayload::map_project()`] or one of its sister methods.
 ///
-/// ## Downcasting from a trait object
-///
-/// If you have a [`DataPayload`]`<`[`ErasedDataStructMarker`]`>`, use [`DataPayload::downcast()`]
-/// to transform it into a payload of a concrete type.
-///
 /// # Examples
 ///
 /// Basic usage, using the `CowStrMarker` marker:
@@ -150,18 +145,16 @@ where
 ///
 /// assert_eq!("Demo", payload.get());
 /// ```
-///
-/// [`ErasedDataStructMarker`]: crate::erased::ErasedDataStructMarker
-pub struct DataPayload<'data, M>
+pub struct DataPayload<M>
 where
-    M: DataMarker<'data>,
+    M: DataMarker,
 {
-    pub(crate) inner: DataPayloadInner<'data, M>,
+    pub(crate) yoke: Yoke<M::Yokeable, Option<Rc<[u8]>>>,
 }
 
-impl<'data, M> Debug for DataPayload<'data, M>
+impl<M> Debug for DataPayload<M>
 where
-    M: DataMarker<'data>,
+    M: DataMarker,
     for<'a> &'a <M::Yokeable as Yokeable<'a>>::Output: Debug,
 {
     fn fmt(&self, f: &mut core::fmt::Formatter<'_>) -> core::fmt::Result {
@@ -181,25 +174,21 @@ where
 /// let resp1: DataPayload<HelloWorldV1Marker> = todo!();
 /// let resp2 = resp1.clone();
 /// ```
-impl<'data, M> Clone for DataPayload<'data, M>
+impl<M> Clone for DataPayload<M>
 where
-    M: DataMarker<'data>,
+    M: DataMarker,
     for<'a> YokeTraitHack<<M::Yokeable as Yokeable<'a>>::Output>: Clone,
 {
     fn clone(&self) -> Self {
-        use DataPayloadInner::*;
-        let new_inner = match &self.inner {
-            RcStruct(yoke) => RcStruct(yoke.clone()),
-            Owned(yoke) => Owned(yoke.clone()),
-            RcBuf(yoke) => RcBuf(yoke.clone()),
-        };
-        Self { inner: new_inner }
+        Self {
+            yoke: self.yoke.clone(),
+        }
     }
 }
 
-impl<'data, M> PartialEq for DataPayload<'data, M>
+impl<M> PartialEq for DataPayload<M>
 where
-    M: DataMarker<'data>,
+    M: DataMarker,
     for<'a> YokeTraitHack<<M::Yokeable as Yokeable<'a>>::Output>: PartialEq,
 {
     fn eq(&self, other: &Self) -> bool {
@@ -207,9 +196,9 @@ where
     }
 }
 
-impl<'data, M> Eq for DataPayload<'data, M>
+impl<M> Eq for DataPayload<M>
 where
-    M: DataMarker<'data>,
+    M: DataMarker,
     for<'a> YokeTraitHack<<M::Yokeable as Yokeable<'a>>::Output>: Eq,
 {
 }
@@ -222,46 +211,9 @@ fn test_clone_eq() {
     assert_eq!(p1, p2);
 }
 
-impl<'data, M> DataPayload<'data, M>
+impl<M> DataPayload<M>
 where
-    M: DataMarker<'data>,
-    M::Yokeable: ZeroCopyFrom<M::Cart>,
-{
-    /// Convert an [`Rc`]`<`[`Cart`]`>` into a [`DataPayload`].
-    ///
-    /// The data need not be fully owned; this constructor creates payloads bounded by `'data`.
-    ///
-    /// # Examples
-    ///
-    /// ```
-    /// use icu_provider::prelude::*;
-    /// use icu_provider::hello_world::*;
-    /// use std::borrow::Cow;
-    /// use std::rc::Rc;
-    ///
-    /// let local_data = "example".to_string();
-    ///
-    /// let rc_struct = Rc::from(HelloWorldV1 {
-    ///     message: Cow::Borrowed(&local_data),
-    /// });
-    ///
-    /// let payload = DataPayload::<HelloWorldV1Marker>::from_partial_owned(rc_struct.clone());
-    ///
-    /// assert_eq!(payload.get(), &*rc_struct);
-    /// ```
-    ///
-    /// [`Cart`]: crate::marker::DataMarker::Cart
-    #[inline]
-    pub fn from_partial_owned(data: Rc<M::Cart>) -> Self {
-        Self {
-            inner: DataPayloadInner::RcStruct(Yoke::attach_to_rc_cart(data)),
-        }
-    }
-}
-
-impl<'data, M> DataPayload<'data, M>
-where
-    M: DataMarker<'data>,
+    M: DataMarker,
 {
     /// Convert a byte buffer into a [`DataPayload`]. A function must be provided to perform the
     /// conversion. This can often be a Serde deserialization operation.
@@ -276,10 +228,8 @@ where
         rc_buffer: Rc<[u8]>,
         f: impl for<'de> FnOnce(&'de [u8]) -> Result<<M::Yokeable as Yokeable<'de>>::Output, E>,
     ) -> Result<Self, E> {
-        let yoke = Yoke::try_attach_to_cart(rc_buffer, f)?;
-        Ok(Self {
-            inner: DataPayloadInner::RcBuf(yoke),
-        })
+        let yoke = Yoke::try_attach_to_cart(rc_buffer, f)?.wrap_cart_in_option();
+        Ok(Self { yoke })
     }
 
     /// Convert a byte buffer into a [`DataPayload`]. A function must be provided to perform the
@@ -293,7 +243,7 @@ where
     /// # Examples
     ///
     /// ```
-    /// # #[cfg(feature = "provider_serde")] {
+    /// # #[cfg(feature = "serde_json")] {
     /// use icu_provider::prelude::*;
     /// use icu_provider::hello_world::*;
     /// use std::rc::Rc;
@@ -310,17 +260,15 @@ where
     /// .expect("JSON is valid");
     ///
     /// assert_eq!("Hello World", payload.get().message);
-    /// # } // feature = "provider_serde"
+    /// # } // feature = "serde_json"
     /// ```
     #[allow(clippy::type_complexity)]
     pub fn try_from_rc_buffer_badly<E>(
         rc_buffer: Rc<[u8]>,
         f: for<'de> fn(&'de [u8]) -> Result<<M::Yokeable as Yokeable<'de>>::Output, E>,
     ) -> Result<Self, E> {
-        let yoke = Yoke::try_attach_to_cart_badly(rc_buffer, f)?;
-        Ok(Self {
-            inner: DataPayloadInner::RcBuf(yoke),
-        })
+        let yoke = Yoke::try_attach_to_cart(rc_buffer, f)?.wrap_cart_in_option();
+        Ok(Self { yoke })
     }
 
     /// Convert a byte buffer into a [`DataPayload`]. A function must be provided to perform the
@@ -332,7 +280,7 @@ where
     /// # Examples
     ///
     /// ```
-    /// # #[cfg(feature = "provider_serde")] {
+    /// # #[cfg(feature = "serde_json")] {
     /// use icu_provider::prelude::*;
     /// use icu_provider::hello_world::*;
     /// use std::rc::Rc;
@@ -342,7 +290,7 @@ where
     /// let json_rc_buffer: Rc<[u8]> = json_text.as_bytes().into();
     ///
     /// let payload = DataPayload::<HelloWorldV1Marker>::try_from_yoked_buffer(
-    ///     Yoke::attach_to_rc_cart(json_rc_buffer),
+    ///     Yoke::attach_to_zero_copy_cart(json_rc_buffer),
     ///     (),
     ///     |bytes, _, _| {
     ///         serde_json::from_slice(bytes)
@@ -351,7 +299,7 @@ where
     /// .expect("JSON is valid");
     ///
     /// assert_eq!("Hello World", payload.get().message);
-    /// # } // feature = "provider_serde"
+    /// # } // feature = "serde_json"
     /// ```
     #[allow(clippy::type_complexity)]
     pub fn try_from_yoked_buffer<T, E>(
@@ -363,10 +311,10 @@ where
             PhantomData<&'de ()>,
         ) -> Result<<M::Yokeable as Yokeable<'de>>::Output, E>,
     ) -> Result<Self, E> {
-        let yoke = yoked_buffer.try_project_with_capture(capture, f)?;
-        Ok(Self {
-            inner: DataPayloadInner::RcBuf(yoke),
-        })
+        let yoke = yoked_buffer
+            .wrap_cart_in_option()
+            .try_project_with_capture(capture, f)?;
+        Ok(Self { yoke })
     }
 
     /// Convert a fully owned (`'static`) data struct into a DataPayload.
@@ -391,10 +339,18 @@ where
     #[inline]
     pub fn from_owned(data: M::Yokeable) -> Self {
         Self {
-            inner: DataPayloadInner::Owned(Yoke::new_always_owned(data)),
+            yoke: Yoke::new_owned(data),
         }
     }
 
+    /// Convert a DataPayload that was created via [`DataPayload::from_owned()`] back into the
+    /// concrete type used to construct it.
+    pub fn try_unwrap_owned(self) -> Result<M::Yokeable, DataError> {
+        self.yoke
+            .try_into_yokeable()
+            .map_err(|_| DataErrorKind::InvalidState.with_str_context("try_unwrap_owned"))
+    }
+
     /// Mutate the data contained in this DataPayload.
     ///
     /// For safety, all mutation operations must take place within a helper function that cannot
@@ -432,12 +388,7 @@ where
     where
         F: 'static + for<'b> FnOnce(&'b mut <M::Yokeable as Yokeable<'a>>::Output),
     {
-        use DataPayloadInner::*;
-        match &mut self.inner {
-            RcStruct(yoke) => yoke.with_mut(f),
-            Owned(yoke) => yoke.with_mut(f),
-            RcBuf(yoke) => yoke.with_mut(f),
-        }
+        self.yoke.with_mut(f)
     }
 
     /// Borrows the underlying data.
@@ -455,14 +406,10 @@ where
     ///
     /// assert_eq!("Demo", payload.get());
     /// ```
+    #[inline]
     #[allow(clippy::needless_lifetimes)]
     pub fn get<'a>(&'a self) -> &'a <M::Yokeable as Yokeable<'a>>::Output {
-        use DataPayloadInner::*;
-        match &self.inner {
-            RcStruct(yoke) => yoke.get(),
-            Owned(yoke) => yoke.get(),
-            RcBuf(yoke) => yoke.get(),
-        }
+        self.yoke.get()
     }
 
     /// Maps `DataPayload<M>` to `DataPayload<M2>` by projecting it with [`Yoke::project`].
@@ -471,9 +418,6 @@ where
     /// type. The function takes a second argument which should be ignored. For more details,
     /// see [`Yoke::project()`].
     ///
-    /// Both `M` and `M2` have the same [`DataMarker::Cart`]. This means that when using
-    /// `map_project`, it is usually necessary to define a custom [`DataMarker`] type.
-    ///
     /// The standard [`DataPayload::map_project()`] function moves `self` and cannot capture any
     /// data from its context. Use one of the sister methods if you need these capabilities:
     ///
@@ -494,9 +438,8 @@ where
     /// // target type, and the Cart should correspond to the type being transformed.
     ///
     /// struct HelloWorldV1MessageMarker;
-    /// impl<'data> DataMarker<'data> for HelloWorldV1MessageMarker {
+    /// impl DataMarker for HelloWorldV1MessageMarker {
     ///     type Yokeable = Cow<'static, str>;
-    ///     type Cart = HelloWorldV1<'data>;
     /// }
     ///
     /// let p1: DataPayload<HelloWorldV1Marker> = DataPayload::from_owned(HelloWorldV1 {
@@ -519,21 +462,12 @@ where
             <M::Yokeable as Yokeable<'a>>::Output,
             PhantomData<&'a ()>,
         ) -> <M2::Yokeable as Yokeable<'a>>::Output,
-    ) -> DataPayload<'data, M2>
+    ) -> DataPayload<M2>
     where
-        M2: DataMarker<'data, Cart = M::Cart>,
+        M2: DataMarker,
     {
-        use DataPayloadInner::*;
-        match self.inner {
-            RcStruct(yoke) => DataPayload {
-                inner: RcStruct(yoke.project(f)),
-            },
-            Owned(yoke) => DataPayload {
-                inner: Owned(yoke.project(f)),
-            },
-            RcBuf(yoke) => DataPayload {
-                inner: RcBuf(yoke.project(f)),
-            },
+        DataPayload {
+            yoke: self.yoke.project(f),
         }
     }
 
@@ -552,9 +486,8 @@ where
     /// # use icu_provider::prelude::*;
     /// # use std::borrow::Cow;
     /// # struct HelloWorldV1MessageMarker;
-    /// # impl<'data> DataMarker<'data> for HelloWorldV1MessageMarker {
+    /// # impl DataMarker for HelloWorldV1MessageMarker {
     /// #     type Yokeable = Cow<'static, str>;
-    /// #     type Cart = HelloWorldV1<'data>;
     /// # }
     ///
     /// let p1: DataPayload<HelloWorldV1Marker> = DataPayload::from_owned(HelloWorldV1 {
@@ -577,21 +510,12 @@ where
             &'this <M::Yokeable as Yokeable<'a>>::Output,
             PhantomData<&'a ()>,
         ) -> <M2::Yokeable as Yokeable<'a>>::Output,
-    ) -> DataPayload<'data, M2>
+    ) -> DataPayload<M2>
     where
-        M2: DataMarker<'data, Cart = M::Cart>,
+        M2: DataMarker,
     {
-        use DataPayloadInner::*;
-        match &self.inner {
-            RcStruct(yoke) => DataPayload {
-                inner: RcStruct(yoke.project_cloned(f)),
-            },
-            Owned(yoke) => DataPayload {
-                inner: Owned(yoke.project_cloned(f)),
-            },
-            RcBuf(yoke) => DataPayload {
-                inner: RcBuf(yoke.project_cloned(f)),
-            },
+        DataPayload {
+            yoke: self.yoke.project_cloned(f),
         }
     }
 
@@ -611,9 +535,8 @@ where
     /// # use icu_provider::prelude::*;
     /// # use std::borrow::Cow;
     /// # struct HelloWorldV1MessageMarker;
-    /// # impl<'data> DataMarker<'data> for HelloWorldV1MessageMarker {
+    /// # impl DataMarker for HelloWorldV1MessageMarker {
     /// #     type Yokeable = Cow<'static, str>;
-    /// #     type Cart = HelloWorldV1<'data>;
     /// # }
     ///
     /// let p1: DataPayload<HelloWorldV1Marker> = DataPayload::from_owned(HelloWorldV1 {
@@ -640,9 +563,8 @@ where
     /// # use icu_provider::prelude::*;
     /// # use std::borrow::Cow;
     /// # struct HelloWorldV1MessageMarker;
-    /// # impl<'data> DataMarker<'data> for HelloWorldV1MessageMarker {
+    /// # impl DataMarker for HelloWorldV1MessageMarker {
     /// #     type Yokeable = Cow<'static, str>;
-    /// #     type Cart = HelloWorldV1<'data>;
     /// # }
     ///
     /// let p1: DataPayload<HelloWorldV1Marker> = DataPayload::from_owned(HelloWorldV1 {
@@ -669,21 +591,12 @@ where
             capture: T,
             PhantomData<&'a ()>,
         ) -> <M2::Yokeable as Yokeable<'a>>::Output,
-    ) -> DataPayload<'data, M2>
+    ) -> DataPayload<M2>
     where
-        M2: DataMarker<'data, Cart = M::Cart>,
+        M2: DataMarker,
     {
-        use DataPayloadInner::*;
-        match self.inner {
-            RcStruct(yoke) => DataPayload {
-                inner: RcStruct(yoke.project_with_capture(capture, f)),
-            },
-            Owned(yoke) => DataPayload {
-                inner: Owned(yoke.project_with_capture(capture, f)),
-            },
-            RcBuf(yoke) => DataPayload {
-                inner: RcBuf(yoke.project_with_capture(capture, f)),
-            },
+        DataPayload {
+            yoke: self.yoke.project_with_capture(capture, f),
         }
     }
 
@@ -703,9 +616,8 @@ where
     /// # use icu_provider::prelude::*;
     /// # use std::borrow::Cow;
     /// # struct HelloWorldV1MessageMarker;
-    /// # impl<'data> DataMarker<'data> for HelloWorldV1MessageMarker {
+    /// # impl DataMarker for HelloWorldV1MessageMarker {
     /// #     type Yokeable = Cow<'static, str>;
-    /// #     type Cart = HelloWorldV1<'data>;
     /// # }
     ///
     /// let p1: DataPayload<HelloWorldV1Marker> = DataPayload::from_owned(HelloWorldV1 {
@@ -735,21 +647,12 @@ where
             capture: T,
             PhantomData<&'a ()>,
         ) -> <M2::Yokeable as Yokeable<'a>>::Output,
-    ) -> DataPayload<'data, M2>
+    ) -> DataPayload<M2>
     where
-        M2: DataMarker<'data, Cart = M::Cart>,
+        M2: DataMarker,
     {
-        use DataPayloadInner::*;
-        match &self.inner {
-            RcStruct(yoke) => DataPayload {
-                inner: RcStruct(yoke.project_cloned_with_capture(capture, f)),
-            },
-            Owned(yoke) => DataPayload {
-                inner: Owned(yoke.project_cloned_with_capture(capture, f)),
-            },
-            RcBuf(yoke) => DataPayload {
-                inner: RcBuf(yoke.project_cloned_with_capture(capture, f)),
-            },
+        DataPayload {
+            yoke: self.yoke.project_cloned_with_capture(capture, f),
         }
     }
 
@@ -769,9 +672,8 @@ where
     /// # use icu_provider::prelude::*;
     /// # use std::borrow::Cow;
     /// # struct HelloWorldV1MessageMarker;
-    /// # impl<'data> DataMarker<'data> for HelloWorldV1MessageMarker {
+    /// # impl DataMarker for HelloWorldV1MessageMarker {
     /// #     type Yokeable = Cow<'static, str>;
-    /// #     type Cart = HelloWorldV1<'data>;
     /// # }
     ///
     /// let p1: DataPayload<HelloWorldV1Marker> = DataPayload::from_owned(HelloWorldV1 {
@@ -802,9 +704,8 @@ where
     /// # use icu_provider::prelude::*;
     /// # use std::borrow::Cow;
     /// # struct HelloWorldV1MessageMarker;
-    /// # impl<'data> DataMarker<'data> for HelloWorldV1MessageMarker {
+    /// # impl DataMarker for HelloWorldV1MessageMarker {
     /// #     type Yokeable = Cow<'static, str>;
-    /// #     type Cart = HelloWorldV1<'data>;
     /// # }
     ///
     /// let p1: DataPayload<HelloWorldV1Marker> = DataPayload::from_owned(HelloWorldV1 {
@@ -835,21 +736,12 @@ where
             capture: T,
             PhantomData<&'a ()>,
         ) -> Result<<M2::Yokeable as Yokeable<'a>>::Output, E>,
-    ) -> Result<DataPayload<'data, M2>, E>
+    ) -> Result<DataPayload<M2>, E>
     where
-        M2: DataMarker<'data, Cart = M::Cart>,
+        M2: DataMarker,
     {
-        use DataPayloadInner::*;
-        Ok(match self.inner {
-            RcStruct(yoke) => DataPayload {
-                inner: RcStruct(yoke.try_project_with_capture(capture, f)?),
-            },
-            Owned(yoke) => DataPayload {
-                inner: Owned(yoke.try_project_with_capture(capture, f)?),
-            },
-            RcBuf(yoke) => DataPayload {
-                inner: RcBuf(yoke.try_project_with_capture(capture, f)?),
-            },
+        Ok(DataPayload {
+            yoke: self.yoke.try_project_with_capture(capture, f)?,
         })
     }
 
@@ -869,9 +761,8 @@ where
     /// # use icu_provider::prelude::*;
     /// # use std::borrow::Cow;
     /// # struct HelloWorldV1MessageMarker;
-    /// # impl<'data> DataMarker<'data> for HelloWorldV1MessageMarker {
+    /// # impl DataMarker for HelloWorldV1MessageMarker {
     /// #     type Yokeable = Cow<'static, str>;
-    /// #     type Cart = HelloWorldV1<'data>;
     /// # }
     ///
     /// let p1: DataPayload<HelloWorldV1Marker> = DataPayload::from_owned(HelloWorldV1 {
@@ -905,62 +796,100 @@ where
             capture: T,
             PhantomData<&'a ()>,
         ) -> Result<<M2::Yokeable as Yokeable<'a>>::Output, E>,
-    ) -> Result<DataPayload<'data, M2>, E>
+    ) -> Result<DataPayload<M2>, E>
     where
-        M2: DataMarker<'data, Cart = M::Cart>,
+        M2: DataMarker,
     {
-        use DataPayloadInner::*;
-        Ok(match &self.inner {
-            RcStruct(yoke) => DataPayload {
-                inner: RcStruct(yoke.try_project_cloned_with_capture(capture, f)?),
-            },
-            Owned(yoke) => DataPayload {
-                inner: Owned(yoke.try_project_cloned_with_capture(capture, f)?),
-            },
-            RcBuf(yoke) => DataPayload {
-                inner: RcBuf(yoke.try_project_cloned_with_capture(capture, f)?),
-            },
+        Ok(DataPayload {
+            yoke: self.yoke.try_project_cloned_with_capture(capture, f)?,
         })
     }
 }
 
-/// A response object containing an object as payload and metadata about it.
-pub struct DataResponse<'data, M>
+impl DataPayload<BufferMarker> {
+    /// Converts a reference-counted byte buffer into a `DataPayload<BufferMarker>`.
+    pub fn from_rc_buffer(buffer: Rc<[u8]>) -> Self {
+        Self {
+            yoke: Yoke::attach_to_zero_copy_cart(buffer).wrap_cart_in_option(),
+        }
+    }
+
+    /// Converts a yoked byte buffer into a `DataPayload<BufferMarker>`.
+    pub fn from_yoked_buffer(yoked_buffer: Yoke<&'static [u8], Rc<[u8]>>) -> Self {
+        Self {
+            yoke: yoked_buffer.wrap_cart_in_option(),
+        }
+    }
+
+    /// Converts a static byte buffer into a `DataPayload<BufferMarker>`.
+    pub fn from_static_buffer(buffer: &'static [u8]) -> Self {
+        Self {
+            yoke: Yoke::new_owned(buffer),
+        }
+    }
+}
+
+impl<M> Default for DataPayload<M>
 where
-    M: DataMarker<'data>,
+    M: DataMarker,
+    M::Yokeable: Default,
+{
+    fn default() -> Self {
+        Self::from_owned(Default::default())
+    }
+}
+
+/// A response object containing an object as payload and metadata about it.
+pub struct DataResponse<M>
+where
+    M: DataMarker,
 {
     /// Metadata about the returned object.
     pub metadata: DataResponseMetadata,
 
     /// The object itself; None if it was not loaded.
-    pub payload: Option<DataPayload<'data, M>>,
+    pub payload: Option<DataPayload<M>>,
 }
 
-impl<'data, M> DataResponse<'data, M>
+impl<M> DataResponse<M>
 where
-    M: DataMarker<'data>,
+    M: DataMarker,
 {
     /// Takes ownership of the underlying payload. Error if not present.
+    ///
+    /// To take the metadata, too, use [`Self::take_metadata_and_payload()`].
     #[inline]
-    pub fn take_payload(self) -> Result<DataPayload<'data, M>, Error> {
-        self.payload.ok_or(Error::MissingPayload)
+    pub fn take_payload(self) -> Result<DataPayload<M>, DataError> {
+        Ok(self.take_metadata_and_payload()?.1)
+    }
+
+    /// Takes ownership of the underlying metadata and payload. Error if payload is not present.
+    #[inline]
+    pub fn take_metadata_and_payload(
+        self,
+    ) -> Result<(DataResponseMetadata, DataPayload<M>), DataError> {
+        Ok((
+            self.metadata,
+            self.payload
+                .ok_or_else(|| DataErrorKind::MissingPayload.with_type_context::<M>())?,
+        ))
     }
 }
 
-impl<'data, M> TryFrom<DataResponse<'data, M>> for DataPayload<'data, M>
+impl<M> TryFrom<DataResponse<M>> for DataPayload<M>
 where
-    M: DataMarker<'data>,
+    M: DataMarker,
 {
-    type Error = Error;
+    type Error = DataError;
 
-    fn try_from(response: DataResponse<'data, M>) -> Result<Self, Self::Error> {
+    fn try_from(response: DataResponse<M>) -> Result<Self, Self::Error> {
         response.take_payload()
     }
 }
 
-impl<'data, M> Debug for DataResponse<'data, M>
+impl<M> Debug for DataResponse<M>
 where
-    M: DataMarker<'data>,
+    M: DataMarker,
     for<'a> &'a <M::Yokeable as Yokeable<'a>>::Output: Debug,
 {
     fn fmt(&self, f: &mut core::fmt::Formatter<'_>) -> core::fmt::Result {
@@ -984,9 +913,9 @@ where
 /// let resp1: DataResponse<HelloWorldV1Marker> = todo!();
 /// let resp2 = resp1.clone();
 /// ```
-impl<'data, M> Clone for DataResponse<'data, M>
+impl<M> Clone for DataResponse<M>
 where
-    M: DataMarker<'data>,
+    M: DataMarker,
     for<'a> YokeTraitHack<<M::Yokeable as Yokeable<'a>>::Output>: Clone,
 {
     fn clone(&self) -> Self {
@@ -1007,7 +936,7 @@ fn test_debug() {
             message: Cow::Borrowed("foo"),
         })),
     };
-    assert_eq!("DataResponse { metadata: DataResponseMetadata { data_langid: None }, payload: Some(HelloWorldV1 { message: \"foo\" }) }", format!("{:?}", resp));
+    assert_eq!("DataResponse { metadata: DataResponseMetadata { data_langid: None, buffer_format: None }, payload: Some(HelloWorldV1 { message: \"foo\" }) }", format!("{:?}", resp));
 }
 
 /// A generic data provider that loads a payload of a specific type.
@@ -1015,15 +944,15 @@ fn test_debug() {
 /// See examples on some of the concrete implementations:
 ///
 /// - [`HelloWorldProvider`](crate::hello_world::HelloWorldProvider)
-/// - [`StructProvider`](crate::struct_provider::StructProvider)
+/// - [`AnyPayloadProvider`](crate::struct_provider::AnyPayloadProvider)
 /// - [`InvariantDataProvider`](crate::inv::InvariantDataProvider)
-pub trait DataProvider<'data, M>
+pub trait DataProvider<M>
 where
-    M: DataMarker<'data>,
+    M: DataMarker,
 {
     /// Query the provider for data, returning the result.
     ///
     /// Returns [`Ok`] if the request successfully loaded data. If data failed to load, returns an
     /// Error with more information.
-    fn load_payload(&self, req: &DataRequest) -> Result<DataResponse<'data, M>, Error>;
+    fn load_payload(&self, req: &DataRequest) -> Result<DataResponse<M>, DataError>;
 }
