@@ -4,57 +4,54 @@
 
 //! Utilities for using trait objects with `DataPayload`.
 
-/// Trait to allow conversion from `DataPayload<T>` to `DataPayload<S>` where `T` implements `S`.
+/// Trait to allow conversion from `DataPayload<T>` to `DataPayload<S>`.
 ///
-/// This is used internally by [`impl_dyn_provider!`] and is not intended to be called from userland
-/// code. You may be looking for [`DataPayload::downcast`], which converts in the other direction.
+/// This trait can be manually implemented in order to enable [`impl_dyn_provider`].
 ///
 /// [`DataPayload::downcast`]: crate::DataPayload::downcast
-pub trait UpcastDataPayload<'data, M>
+pub trait UpcastDataPayload<M>
 where
-    M: crate::prelude::DataMarker<'data>,
-    Self: Sized + crate::prelude::DataMarker<'data>,
+    M: crate::prelude::DataMarker,
+    Self: Sized + crate::prelude::DataMarker,
 {
     /// Upcast a `DataPayload<T>` to a `DataPayload<S>` where `T` implements trait `S`.
     ///
     /// # Examples
     ///
     /// Upcast and then downcast a data struct of type `Cow<str>` (cart type `String`) via
-    /// [`ErasedDataStruct`](crate::erased::ErasedDataStruct):
+    /// [`AnyPayload`](crate::any::AnyPayload):
     ///
     /// ```
     /// use icu_provider::prelude::*;
-    /// use icu_provider::erased::*;
     /// use icu_provider::dynutil::UpcastDataPayload;
-    /// use icu_provider::marker::CowStringMarker;
+    /// use icu_provider::marker::CowStrMarker;
     /// use std::borrow::Cow;
     ///
     /// let data = "foo".to_string();
-    /// let original = DataPayload::<CowStringMarker>::from_owned(Cow::Owned(data));
-    /// let upcasted = ErasedDataStructMarker::upcast(original);
+    /// let original = DataPayload::<CowStrMarker>::from_owned(Cow::Owned(data));
+    /// let upcasted = AnyMarker::upcast(original);
     /// let downcasted = upcasted
-    ///     .downcast::<CowStringMarker>()
+    ///     .downcast::<CowStrMarker>()
     ///     .expect("Type conversion");
     /// assert_eq!(downcasted.get(), "foo");
     /// ```
-    fn upcast(
-        other: crate::prelude::DataPayload<'data, M>,
-    ) -> crate::prelude::DataPayload<'data, Self>;
+    fn upcast(other: crate::prelude::DataPayload<M>) -> crate::prelude::DataPayload<Self>;
 }
 
-/// Implement [`DataProvider`] for a trait object `S` on a type that already implements [`DataProvider`]
-/// for one or more `M`, where `M` is a concrete type that implements the trait `S`.
+/// Implements [`DataProvider`] for a marker type `S` on a type that already implements
+/// [`DataProvider`] for one or more `M`, where `M` is a concrete type that is convertible to `S`
+/// via [`UpcastDataPayload`].
 ///
 /// Use this macro to add support to your data provider for:
 ///
-/// - [`ErasedDataStruct`] if your provider can return typed objects as [`Any`](core::any::Any)
-/// - [`SerdeSeDataStruct`] if your provider returns objects implementing [`serde::Serialize`]
+/// - [`AnyPayload`] if your provider can return typed objects as [`Any`](core::any::Any)
+/// - [`SerializeMarker`] if your provider returns objects implementing [`serde::Serialize`]
 ///
 /// The second argument is a match-like construction mapping from resource keys to structs. To map
 /// multiple keys to a single data struct, use `_` as the data key.
 ///
-/// The third argument can be either the trait object marker, like [`SerdeSeDataStructMarker`], or the
-/// shorthands `ERASED` or `SERDE_SE`.
+/// The third argument can be either the trait object marker, like [`SerializeMarker`], or the
+/// shorthands `ANY` or `SERDE_SE`.
 ///
 /// Lifetimes:
 ///
@@ -66,16 +63,15 @@ where
 ///
 /// ```
 /// use icu_provider::prelude::*;
-/// use icu_provider::erased::ErasedDataStructMarker;
-/// use icu_provider::marker::CowStringMarker;
+/// use icu_provider::marker::CowStrMarker;
 /// use std::borrow::Cow;
-/// const DEMO_KEY: ResourceKey = icu_provider::resource_key!(x, "foo", "bar", 1);
+/// const DEMO_KEY: ResourceKey = icu_provider::resource_key!("foo/bar@1");
 ///
 /// // A small DataProvider that returns owned strings
 /// struct MyProvider(pub String);
-/// impl<'data> DataProvider<'static, CowStringMarker> for MyProvider {
+/// impl DataProvider<CowStrMarker> for MyProvider {
 ///     fn load_payload(&self, req: &DataRequest)
-///             -> Result<DataResponse<'static, CowStringMarker>, DataError> {
+///             -> Result<DataResponse<CowStrMarker>, DataError> {
 ///         req.resource_path.key.match_key(DEMO_KEY)?;
 ///         Ok(DataResponse {
 ///             metadata: Default::default(),
@@ -84,17 +80,17 @@ where
 ///     }
 /// }
 ///
-/// // Implement DataProvider<ErasedDataStructMarker>
+/// // Implement DataProvider<AnyMarker>
 /// icu_provider::impl_dyn_provider!(MyProvider, {
-///     DEMO_KEY => CowStringMarker,
-/// }, ERASED);
+///     DEMO_KEY => CowStrMarker,
+/// }, ANY);
 ///
 /// // Usage example
 /// let provider = MyProvider("demo".to_string());
-/// let resp: DataResponse<ErasedDataStructMarker> = provider
+/// let resp: DataResponse<AnyMarker> = provider
 ///     .load_payload(&DEMO_KEY.into())
 ///     .expect("Loading should succeed");
-/// let payload: DataPayload<CowStringMarker> = resp
+/// let payload: DataPayload<CowStrMarker> = resp
 ///     .take_payload()
 ///     .expect("Payload should be present")
 ///     .downcast()
@@ -106,57 +102,52 @@ where
 ///
 /// ```
 /// # use icu_provider::prelude::*;
-/// # use icu_provider::marker::CowStringMarker;
+/// # use icu_provider::marker::CowStrMarker;
 /// # use std::borrow::Cow;
 /// # struct MyProvider(pub String);
-/// # impl<'data> DataProvider<'static, CowStringMarker> for MyProvider {
+/// # impl DataProvider<CowStrMarker> for MyProvider {
 /// #   fn load_payload(&self, req: &DataRequest)
-/// #           -> Result<DataResponse<'static, CowStringMarker>, DataError> {
+/// #           -> Result<DataResponse<CowStrMarker>, DataError> {
 /// #       Ok(DataResponse {
 /// #           metadata: Default::default(),
 /// #           payload: Some(DataPayload::from_owned(self.0.to_string().into()))
 /// #       })
 /// #   }
 /// # }
-/// // Send all keys to the `CowStringMarker` provider.
+/// // Send all keys to the `CowStrMarker` provider.
 /// icu_provider::impl_dyn_provider!(MyProvider, {
-///     _ => CowStringMarker,
-/// }, ERASED);
+///     _ => CowStrMarker,
+/// }, ANY);
 /// ```
 ///
 /// [`DataProvider`]: crate::DataProvider
-/// [`ErasedDataStruct`]: (crate::erased::ErasedDataStruct)
-/// [`SerdeSeDataStruct`]: (crate::serde::SerdeSeDataStruct)
-/// [`SerdeSeDataStructMarker`]: (crate::serde::SerdeSeDataStructMarker)
+/// [`AnyPayload`]: (crate::any::AnyPayload)
+/// [`SerializeMarker`]: (crate::serde::SerializeMarker)
 #[macro_export]
 macro_rules! impl_dyn_provider {
-    ($provider:ty, { $($pat:pat => $struct_m:ty),+, }, ERASED) => {
+    ($provider:ty, { $($pat:pat => $struct_m:ty),+, }, ANY) => {
         $crate::impl_dyn_provider!(
             $provider,
             { $($pat => $struct_m),+, },
-            $crate::erased::ErasedDataStructMarker,
-            'data: 'static
+            $crate::any::AnyMarker
         );
     };
-    ($provider:ty, { $($pat:pat => $struct_m:ty),+, }, SERDE_SE, $data:lifetime) => {
-        // If this fails to compile, enable the "provider_serde" feature on this crate.
+    ($provider:ty, { $($pat:pat => $struct_m:ty),+, }, SERDE_SE) => {
+        // If this fails to compile, enable the "serialize" feature on this crate.
         $crate::impl_dyn_provider!(
             $provider,
             { $($pat => $struct_m),+, },
-            $crate::serde::SerdeSeDataStructMarker,
-            $data: $data
+            $crate::serde::SerializeMarker
         );
     };
-    ($provider:ty, { $($pat:pat => $struct_m:ty),+, }, $dyn_m:path, $data:lifetime : $sb:lifetime) => {
-        impl<$data> $crate::prelude::DataProvider<$data, $dyn_m> for $provider
-        where
-            $data: $sb,
+    ($provider:ty, { $($pat:pat => $struct_m:ty),+, }, $dyn_m:path) => {
+        impl $crate::prelude::DataProvider<$dyn_m> for $provider
         {
             fn load_payload(
                 &self,
                 req: &$crate::prelude::DataRequest,
             ) -> Result<
-                $crate::prelude::DataResponse<$data, $dyn_m>,
+                $crate::prelude::DataResponse<$dyn_m>,
                 $crate::prelude::DataError,
             > {
                 match req.resource_path.key {
@@ -174,7 +165,7 @@ macro_rules! impl_dyn_provider {
                     )+,
                     // Don't complain if the call site has its own wildcard match
                     #[allow(unreachable_patterns)]
-                    _ => Err(DataError::MissingResourceKey(req.resource_path.key))
+                    _ => Err($crate::prelude::DataErrorKind::MissingResourceKey.with_req(req))
                 }
             }
         }
