@@ -4,40 +4,35 @@
 
 //! Types having to do with the exporting of data.
 
-use crate::error::Error;
-use crate::iter::IterableDataProvider;
+use crate::iter::IterableProvider;
 use crate::prelude::*;
 
 /// An object capable of serializing data payloads to be read by a [`DataProvider`].
 ///
 /// A [`DataProvider`] by itself is "read-only"; this trait enables it to be "read-write".
-pub trait DataExporter<'data, M>
+pub trait DataExporter<M>
 where
-    M: DataMarker<'data>,
+    M: DataMarker,
 {
     /// Save a `payload` corresponding to the given data request (resource path).
-    fn put_payload(
-        &mut self,
-        req: DataRequest,
-        payload: DataPayload<'data, M>,
-    ) -> Result<(), Error>;
+    fn put_payload(&mut self, req: DataRequest, payload: DataPayload<M>) -> Result<(), DataError>;
 
     /// Function called after a key has been fully dumped into the exporter.
-    fn flush(&mut self) -> Result<(), Error> {
+    fn flush(&mut self) -> Result<(), DataError> {
         Ok(())
     }
 
     /// Function called after all keys have been fully dumped.
-    fn close(&mut self) -> Result<(), Error> {
+    fn close(&mut self) -> Result<(), DataError> {
         Ok(())
     }
 }
 
-/// Convenience function to drive a [`DataExporter`] from an [`IterableDataProvider`].
+/// Convenience function to drive a [`DataExporter`] from an [`IterableProvider`].
 ///
 /// # Example
 ///
-/// [`HelloWorldProvider`] implements both [`DataExporter`] and [`IterableDataProvider`]. The
+/// [`HelloWorldProvider`] implements both [`DataExporter`] and [`IterableProvider`]. The
 /// following example copies the data from one instance to another instance.
 ///
 /// ```
@@ -57,18 +52,18 @@ where
 /// ```
 ///
 /// [`HelloWorldProvider`]: crate::hello_world::HelloWorldProvider
-pub fn export_from_iterable<'data, P, E, M>(
+pub fn export_from_iterable<P, E, M>(
     resc_key: &ResourceKey,
     provider: &P,
     exporter: &mut E,
-) -> Result<(), Error>
+) -> Result<(), DataError>
 where
-    M: DataMarker<'data>,
-    P: IterableDataProvider<'data, M> + ?Sized,
-    E: DataExporter<'data, M> + ?Sized,
+    M: DataMarker,
+    P: DataProvider<M> + IterableProvider + ?Sized,
+    E: DataExporter<M> + ?Sized,
 {
     let it = provider.supported_options_for_key(resc_key)?;
-    let try_export = || -> Result<(), Error> {
+    let try_export = || -> Result<(), DataError> {
         for options in it {
             let req = DataRequest {
                 resource_path: ResourcePath {
