@@ -4,12 +4,9 @@
 
 //! Locale-invariant data provider that requires no I/O.
 
-use crate::error::Error;
-use crate::iter::IterableDataProviderCore;
+use crate::iter::IterableProvider;
 use crate::prelude::*;
-use crate::yoke;
 use alloc::boxed::Box;
-use alloc::rc::Rc;
 use alloc::vec;
 use alloc::vec::Vec;
 
@@ -38,27 +35,24 @@ use alloc::vec::Vec;
 /// ```
 pub struct InvariantDataProvider;
 
-impl<'data, M> DataProvider<'data, M> for InvariantDataProvider
+impl<M> DataProvider<M> for InvariantDataProvider
 where
-    M: DataMarker<'data>,
-    M::Cart: Default,
-    M::Yokeable: yoke::ZeroCopyFrom<M::Cart>,
+    M: DataMarker,
+    M::Yokeable: Default,
 {
-    fn load_payload(&self, _req: &DataRequest) -> Result<DataResponse<'data, M>, Error> {
+    fn load_payload(&self, _req: &DataRequest) -> Result<DataResponse<M>, DataError> {
         Ok(DataResponse {
             metadata: DataResponseMetadata::default(),
-            payload: Some(DataPayload::from_partial_owned(
-                Rc::from(M::Cart::default()),
-            )),
+            payload: Some(DataPayload::from_owned(M::Yokeable::default())),
         })
     }
 }
 
-impl IterableDataProviderCore for InvariantDataProvider {
+impl IterableProvider for InvariantDataProvider {
     fn supported_options_for_key(
         &self,
         _resc_key: &ResourceKey,
-    ) -> Result<Box<dyn Iterator<Item = ResourceOptions>>, Error> {
+    ) -> Result<Box<dyn Iterator<Item = ResourceOptions>>, DataError> {
         let list: Vec<ResourceOptions> = vec![ResourceOptions::default()];
         Ok(Box::new(list.into_iter()))
     }
