@@ -5,28 +5,17 @@
 use super::*;
 use crate::prelude::*;
 use alloc::borrow::Cow;
-use alloc::string::String;
 
 /// Marker type for [`Cow`]`<str>` where the backing cart is `str`.
 pub struct CowStrMarker;
 
-impl<'data> DataMarker<'data> for CowStrMarker {
+impl DataMarker for CowStrMarker {
     type Yokeable = Cow<'static, str>;
-    type Cart = str;
 }
 
-impl DataPayload<'static, CowStrMarker> {
+impl DataPayload<CowStrMarker> {
     /// Make a [`DataPayload`]`<`[`CowStrMarker`]`>` from a static string slice.
-    pub fn from_static_str(s: &'static str) -> DataPayload<'static, CowStrMarker> {
+    pub fn from_static_str(s: &'static str) -> DataPayload<CowStrMarker> {
         DataPayload::from_owned(Cow::Borrowed(s))
     }
 }
-
-/// Marker type for [`Cow`]`<str>` where the backing cart is `String`. This is required if
-/// `ErasedDataStruct` is to be used.
-pub struct CowStringMarker;
-
-impl<'data> DataMarker<'data> for CowStringMarker {
-    type Yokeable = Cow<'static, str>;
-    type Cart = String;
-}
