@@ -5,7 +5,7 @@
 use icu_locid::LanguageIdentifier;
 use icu_locid_macros::langid;
 use icu_provider::hello_world::*;
-use icu_provider::iter::IterableDataProviderCore;
+use icu_provider::iter::IterableProvider;
 
 #[test]
 fn test_supported_langids() {
@@ -31,6 +31,7 @@ fn test_supported_langids() {
             langid!("is"),
             langid!("ja"),
             langid!("la"),
+            langid!("pt"),
             langid!("ro"),
             langid!("ru"),
             langid!("vi"),
